package com.codegeist.fisheye.domain.query;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

public class Query {

	private OrderByType orderBy;
	private GroupByType groupBy;
	private String fromDirectory;
	private final List<Restriction> restrictions;
	private final List<ReturnValue> returnValues;
	
	public Query() {
		restrictions = new ArrayList<Restriction>();
		returnValues = new ArrayList<ReturnValue>();
	}
	
	public ReturnValue[] getReturnValues() {
		return returnValues.toArray(new ReturnValue[returnValues.size()]);
	}
	
	public void setFromDirectory(String fromDirectory) {
		this.fromDirectory = fromDirectory;
	}
	
	public String getFromDirectory() {
		return fromDirectory;
	}
	
	public void setOrderBy(OrderByType orderBy) {
		this.orderBy = orderBy;
	}
	
	public void setGroupBy(GroupByType groupBy) {
		this.groupBy = groupBy;
	}
	
	public Query addRestriction(Restriction restriction) {
		if (restriction == null) {
			throw new IllegalArgumentException("Restriction must not be null");
		}
		
		restrictions.add(restriction);
		
		return this;
	}
	
	/**
	 * Adds a new return value to the query.  Duplicate return values will be ignored
	 * @param returnValue Return value to add to the query
	 * @return This query
	 */
	public Query addReturnValue(ReturnValue returnValue) {
		if (returnValue == null) {
			throw new IllegalArgumentException("Return value must not be null");
		}

		// TODO check for duplicates based on type
		returnValues.add(returnValue);
		
		return this;
	}
	
	public String generateEyeQl() {
		StringBuilder sb = new StringBuilder();
		sb.append("select revisions");
		if (fromDirectory != null && fromDirectory.length() > 0) {
			sb.append(" from ");
			sb.append(fromDirectory);
		}
		if (restrictions != null && restrictions.size() > 0) {
			sb.append(" where");
			for (Iterator<Restriction> iRestrictions = restrictions.iterator(); iRestrictions.hasNext();) {
				iRestrictions.next().appendEyeQlFragment(sb);
				if (iRestrictions.hasNext()) {
					sb.append(" and");
				}
			}
		}
		
		if (orderBy != null) {
			sb.append(" order by ");
			sb.append(orderBy.getEyeQl());
		}
		
		if (groupBy != null) {
			sb.append(" group by ");
			sb.append(groupBy.getEyeQl());
		}
		
		if (returnValues != null && returnValues.size() > 0) {
			sb.append(" return ");
			
			for (Iterator<ReturnValue> iReturnValue = returnValues.iterator(); iReturnValue.hasNext(); ) {
				ReturnValue returnValue = iReturnValue.next();
				sb.append(returnValue.attribute.getName());
				if (returnValue.as != null) {
					sb.append(" as ");
					sb.append(returnValue.getAs());
				}
				
				if (iReturnValue.hasNext()) {
					sb.append(", ");
				}
			}
		}
		return sb.toString();
	}
	
	public enum GroupByType {
		FILE("file"), DIR("dir"), CHANGESET("changeset");
		
		private final String eyeql;

		GroupByType(String eyeql) {
			this.eyeql = eyeql;
			
		}
		
		public String getEyeQl() {
			return eyeql;
		}
				
	}
	
	public enum OrderByType {
		DATE("date");
		
		private final String eyeql;

		OrderByType(String eyeql) {
			this.eyeql = eyeql;
			
		}
		
		public String getEyeQl() {
			return eyeql;
		}
				
	}
	
	public static class ReturnValue {
		private final AttributeType attribute;
		private final String as;
		
		public ReturnValue(AttributeType attribute) {
			this(attribute, null);
		}
		
		public ReturnValue(AttributeType attribute, String as) {
			this.attribute = attribute;
			this.as = as;
		}
		
		public AttributeType getAttribute() {
			return attribute;
		}
		
		public String getAs() {
			return as;
		}
		
		public String getColumnName() {
			if (as != null) {
				return as;
			}
			
			return attribute.getName();
		}
	}
	
	public enum AttributeType {
		PATH("path", String.class, new StringConverter()),
		REVISION("revision", String.class, new StringConverter()),
		AUTHOR("author", String.class, new StringConverter()),
		DATE("date", Date.class, new DateConverter()),
		COMMENT("comment", String.class, new StringConverter()),
		CHANGESET("csid", String.class, new StringConverter()),
		IS_BINARY("isBinary", Boolean.class, new BooleanConverter()),
		TOTAL_LINES("totalLines", Integer.class, new IntegerConverter()),
		LINES_ADDED("linesAdded", Integer.class, new IntegerConverter()),
		LINES_REMOVED("linesRemoved", Integer.class, new IntegerConverter()),
		IS_ADDED("isAdded", Boolean.class, new BooleanConverter()),
		IS_DELETED("isDeleted", Boolean.class, new BooleanConverter()),
		IS_COPIED("isCopied", Boolean.class, new BooleanConverter()),
		IS_MOVED("isMoved", Boolean.class, new BooleanConverter()),
		TAGS("tags", String.class, new StringConverter()),
		REVIEWS("reviews", String.class, new StringConverter());
		
		private final String name;
		private final Class<?> javaType;
		private final Converter converter;

		AttributeType(String name, Class<?> javaType, Converter converter) {
			this.name = name;
			this.javaType = javaType;
			this.converter = converter;
			
		}
		
		public String getName() {
			return name;
		}
		
		public Class<?> getJavaType() {
			return javaType;
		}
		
		public Object fromString(String value) {
			return converter.fromString(value);
		}
		
		private static class StringConverter implements Converter {

			public Object fromString(String value) {
				return value;
			}
			
		}
		
		private static class DateConverter implements Converter {
			private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
			
			public Object fromString(String value) {
				try {
					return format.parse(value);
				} catch (ParseException e) {
					// TODO throw a nasty exception
				}
				return null;
			}
			
		}
		
		private static class IntegerConverter implements Converter {

			public Object fromString(String value) {
				return Integer.valueOf(value);
			}
			
		}
		
		private static class BooleanConverter implements Converter {

			public Object fromString(String value) {
				return Boolean.valueOf(value);
			}
			
		}
		
		private interface Converter {
			public abstract Object fromString(String value);
		}
	}
}
