package com.codegeist.fisheye.service;

import java.util.Date;

import com.codegeist.fisheye.domain.Changeset;
import com.codegeist.fisheye.domain.Changesets;
import com.codegeist.fisheye.domain.PathHistory;
import com.codegeist.fisheye.domain.PathInfo;
import com.codegeist.fisheye.domain.Revision;
import com.codegeist.fisheye.domain.Version;
import com.codegeist.fisheye.domain.query.IQueryMatchListener;
import com.codegeist.fisheye.domain.query.Query;

/**
 * <p>
 * Service interface for FishEye 1.4 / Crucible 1.2
 * </p>
 * <p>
 * All services that take an auth token accept 'null', indicating that they work as an anonymous user
 * <p>
 * 
 * @author Brock Janiczak
 */
public interface FisheyeService {
	
	/**
	 * Login and create an authentication token. Returns the token if login was successful, or returns an error otherwise.
	 * @param username Requested username
	 * @param password Requested password
	 * @return Authentication token used for subsequent calls
	 */
	public abstract String login(String username, String password) throws FisheyeException;
	
	/**
	 * Disables the given auth token. Returns true in all cases.
	 * @param auth Authentication token
	 * @return <code>true</code>
	 */
	public abstract boolean logout(String auth) throws FisheyeException;
	
	/**
	 * @return the version number of this FishEye instance.
	 * @since FishEye 1.4 / Crucible 1.2
	 */
	public abstract Version fisheyeVersion() throws FisheyeException;
	
	/**
	 * @return the Crucible version number if Crucible is installed. This API method will return an empty String if this isn't a Crucible instance.
	 * @since FishEye 1.4 / Crucible 1.2
	 */
	public abstract Version crucibleVersion() throws FisheyeException;
	
	/**
	 * @param auth Valid authentication token, or <code>null</code> if the session is anonymous
	 * @return a list of repository names in this FishEye instance.
	 */
	public abstract String[] listRepositories(String auth) throws FisheyeException;
	
	/**
	 * @param auth Valid authentication token, or <code>null</code> if the session is anonymous
	 * @param rep Repository name
	 * @param path Absolute directory or file name.  Use '/' for the root directory of the repository 
	 * @return a list of paths immediately under the given path. A path represents either a file or a directory.
	 */
	public abstract PathInfo[] listPaths(String auth, String rep, String path) throws FisheyeException;
	
	/**
	 * @param auth Valid authentication token, or <code>null</code> if the session is anonymous
	 * @param rep Repository name
	 * @param path Absolute file or directory name.
	 * @param rev Revision
	 * @return details of a particular revision.
	 */
	public abstract Revision getRevision(String auth, String rep, String path, String rev) throws FisheyeException;
	
	/**
	 * @param auth Valid authentication token, or <code>null</code> if the session is anonymous
	 * @param rep Repository name
	 * @param path Absolute file or directory name.
	 * @param rev Revision
	 * @return the tags associated with particular revision as an array of strings.
	 */
	public abstract String[] listTagsForRevision(String auth, String rep, String path, String rev) throws FisheyeException;
	
	/**
	 * @param auth Valid authentication token, or <code>null</code> if the session is anonymous
	 * @param rep Repository name
	 * @param path Absolute file or directory name.
	 * @return history of a particular path.
	 */
	public abstract PathHistory listPathHistory(String auth, String rep, String path) throws FisheyeException;
	
	/**
	 * @param auth Valid authentication token, or <code>null</code> if the session is anonymous
	 * @param rep Repository name
	 * @param csid Changeset ID
	 * @return the details of a particular changeset
	 */
	public abstract Changeset getChangeset(String auth, String rep, String csid) throws FisheyeException;
	
	/**
	 * Lists changes under a given path, optionally between two dates. Returned structure contains a list of changeset ids, from most-recent to least-recent.
	 * <p>
     * To get changes for the whole repository, use a path of "/"
     * <p>
     * If the start date is not specified, there is no lower bound.
     * <p>
     * If the end date is not specified, "now" is used.
     * <p>
     * FishEye has an internal limit of how many changesets it will return from this method (a few thousand). If this limit is exceeded, the return value will be truncated so that it contains the most-recent changesets. This value of this limit is contained in the return value.
	 * @param auth Valid authentication token, or <code>null</code> if the session is anonymous
	 * @param rep Repository name
	 * @param path Absolute file or directory name.
	 * @param start Optional Start Date
	 * @param end Optional End Date
	 * @return Changesets of the supplied path
	 */
	public abstract Changesets listChangesets(String auth, String rep, String path, Date start, Date end) throws FisheyeException;
	

	public void query(String auth, String rep, Query query, IQueryMatchListener reultListener) throws FisheyeException;
}
